@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,32 +29,38 @@
  *  flexible API to let user set attributes for solution selection.
  */
 
-//! HIP = Heterogeneous-compute Interface for Portability
-//!
-//! Define a extremely thin runtime layer that allows source code to be compiled
-//! unmodified through either AMD HCC or NVCC. Key features tend to be in the
-//! spirit and terminology of CUDA, but with a portable path to other
-//! accelerators as well.
-//!
-//! This is the master include file for hipBLASLt, wrapping around rocBLASLt and
-//! cuBLASLt.
-//
-
 #pragma once
 #ifndef _HIPBLASLT_H_
 #define _HIPBLASLT_H_
 
 #include "hipblaslt/hipblaslt-export.h"
 #include "hipblaslt/hipblaslt-version.h"
+#ifndef LEGACY_HIPBLAS_DIRECT
+#include <hipblas-common/hipblas-common.h>
+#else
 #include <hipblas/hipblas.h>
+#endif
 
 #include <memory>
+#include <regex>
 #include <vector>
 
 #include <hip/hip_bfloat16.h>
 #include <hip/hip_complex.h>
 #include <hip/hip_runtime.h>
 #include <hip/hip_runtime_api.h>
+#include <hip/hip_version.h>
+
+#if HIP_VERSION_MAJOR == 6 && HIP_VERSION_MINOR > 2 \
+    && HIP_VERSION_PATCH > 42130 //tmp before gfx94 use hip f8 header
+#define ROCM_USE_FLOAT8 1
+#else
+#undef ROCM_USE_FLOAT8
+#endif
+
+#if defined(__HIPCC__)
+#include <hip/hip_fp8.h>
+#endif
 
 #if defined(__HIP_PLATFORM_AMD__)
 #include "hipblaslt-types.h"
@@ -62,7 +68,12 @@
 
 /* Opaque structures holding information */
 // clang-format off
+
 #define HIPBLASLT_DATATYPE_INVALID static_cast<hipDataType>(255)
+// TODO: Replace static_cast<hipblasComputeType_t>(0) with the appropriate value since 0 represents f16_r
+#define HIPBLASLT_COMPUTE_TYPE_INVALID static_cast<hipblasComputeType_t>(0)
+#define HIPBLASLT_OPERATION_INVALID static_cast<hipblasOperation_t>(0)
+#define ROCBLASLT_COMPUTE_TYPE_INVALID static_cast<rocblaslt_compute_type>(255)
 
 /*! \ingroup types_module
  *  \brief Specify the enum type to set the postprocessing options for the epilogue.
@@ -94,7 +105,7 @@ typedef enum {
    */
   HIPBLASLT_MATRIX_LAYOUT_TYPE = 2,
 
-  /** Memory order of the data, see cublasLtOrder_t.
+  /** Memory order of the data, see hipblasLtOrder_t.
    *
    * int32_t, default: HIPBLASLT_ORDER_COL
    */
@@ -119,7 +130,7 @@ typedef enum {
   /** Matrix leading dimension.
    *
    * For HIPBLASLT_ORDER_COL this is stride (in elements) of matrix column, for more details and documentation for
-   * other memory orders see documentation for cublasLtOrder_t values.
+   * other memory orders see documentation for hipblasLtOrder_t values.
    *
    * Currently only non-negative values are supported, must be large enough so that matrix memory locations are not
    * overlapping (e.g. greater or equal to HIPBLASLT_MATRIX_LAYOUT_ROWS in case of HIPBLASLT_ORDER_COL).
@@ -134,7 +145,8 @@ typedef enum {
  */
 typedef enum {
     HIPBLASLT_POINTER_MODE_HOST = 0,                          /** targets host memory */
-    HIPBLASLT_POINTER_MODE_ALPHA_DEVICE_VECTOR_BETA_HOST = 1, /** alpha pointer targets a device memory vector of length equal to the number of rows of matrix D, and beta is a single value in host memory. */
+    HIPBLASLT_POINTER_MODE_DEVICE = 1,                        /** targets device memory */
+    HIPBLASLT_POINTER_MODE_ALPHA_DEVICE_VECTOR_BETA_HOST = 4, /** alpha pointer targets a device memory vector of length equal to the number of rows of matrix D, and beta is a single value in host memory. */
 } hipblasLtPointerMode_t;
 
 /*! \ingroup types_module
@@ -155,7 +167,12 @@ typedef enum {
   HIPBLASLT_MATMUL_DESC_EPILOGUE_AUX_LD = 11,           /**<The leading dimension of the epilogue auxiliary buffer pointer in the device memory. Data Type:int64_t */
   HIPBLASLT_MATMUL_DESC_EPILOGUE_AUX_BATCH_STRIDE = 12, /**<The batch stride of the epilogue auxiliary buffer pointer in the device memory. Data Type:int64_t */
   HIPBLASLT_MATMUL_DESC_POINTER_MODE = 13,              /**<Specifies alpha and beta are passed by reference, whether they are scalars on the host or on the device, or device vectors. Default value is: HIPBLASLT_POINTER_MODE_HOST (i.e., on the host). Data Type: int32_t based on hipblasLtPointerMode_t*/
-  HIPBLASLT_MATMUL_DESC_MAX = 101
+  HIPBLASLT_MATMUL_DESC_AMAX_D_POINTER = 14,           /**<Device pointer to the memory location that on completion will be set to the maximum of absolute values in the output matrix. Data Type:void* /const void* */
+  HIPBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT = 100,     /**<Compute input A types. Defines the data type used for the input A of matrix multiply. */
+  HIPBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_B_EXT,           /**<Compute input B types. Defines the data type used for the input B of matrix multiply. */
+  HIPBLASLT_MATMUL_DESC_A_SCALE_POINTER_VEC_EXT,        /**<Equivalent to HIPBLASLT_MATMUL_DESC_A_SCALE_POINTER but in vector. Default value: NULL Type: void* /const void* */
+  HIPBLASLT_MATMUL_DESC_B_SCALE_POINTER_VEC_EXT,        /**<Equivalent to HIPBLASLT_MATMUL_DESC_B_SCALE_POINTER but in vector. Default value: NULL Type: void* /const void* */
+  HIPBLASLT_MATMUL_DESC_MAX,
 } hipblasLtMatmulDescAttributes_t;
 
 /*! \ingroup types_module
@@ -191,19 +208,19 @@ typedef enum {
    */
   HIPBLASLT_MATRIX_TRANSFORM_DESC_SCALE_TYPE,
 
-  /** Pointer mode of alpha and beta, see cublasLtPointerMode_t.
+  /** Pointer mode of alpha and beta, see hipblasLtPointerMode_t.
    *
    * int32_t, default: HIPBLASLT_POINTER_MODE_HOST
    */
   HIPBLASLT_MATRIX_TRANSFORM_DESC_POINTER_MODE,
 
-  /** Transform of matrix A, see cublasOperation_t.
+  /** Transform of matrix A, see hipblasOperation_t.
    *
    * int32_t, default: HIPBLAS_OP_N
    */
   HIPBLASLT_MATRIX_TRANSFORM_DESC_TRANSA,
 
-  /** Transform of matrix B, see cublasOperation_t.
+  /** Transform of matrix B, see hipblasOperation_t.
    *
    * int32_t, default: HIPBLAS_OP_N
    */
@@ -226,7 +243,15 @@ typedef struct {
   uint64_t data[8];
 } hipblasLtMatrixTransformDescOpaque_t;
 
-/*! Opaque descriptor for hipblasLtMatrixTransform() operation details
+/*! \ingroup types_module
+ *  \brief Opaque descriptor for hipblasLtMatrixTransform() operation details
+ *
+ *  \details
+ *  The hipblasLtMatrixTransformDesc_t is a pointer to an opaque structure holding the description of a matrix transformation operation.
+ *  \ref hipblasLtMatrixTransformDescCreate():
+ *  To create one instance of the descriptor.
+ *  \ref hipblasLtMatrixTransformDescDestroy():
+ *  To destroy a previously created descriptor and release the resources.
  */
 typedef hipblasLtMatrixTransformDescOpaque_t* hipblasLtMatrixTransformDesc_t;
 /*! \ingroup types_module
@@ -293,11 +318,15 @@ typedef struct _hipblasLtMatmulAlgo_t{
 } hipblasLtMatmulAlgo_t;
 
 /*! \ingroup types_module
+ *  \struct hipblasLtMatmulHeuristicResult_t
  *  \brief Description of the matrix multiplication algorithm
  *
  *  \details
  *  This is a descriptor that holds the configured matrix multiplication algorithm descriptor and its runtime properties.
  *  This structure can be trivially serialized and later restored for use with the same version of hipBLASLt library to save on selecting the right configuration again.
+ *  @param algo \ref hipblasLtMatmulAlgo_t struct
+ *  @param workspaceSize Actual size of workspace memory required
+ *  @param state Result status. Other fields are valid only if, after call to hipblasLtMatmulAlgoGetHeuristic(), this member is set to HIPBLAS_STATUS_SUCCESS
  */
 typedef struct _hipblasLtMatmulHeuristicResult_t{
   hipblasLtMatmulAlgo_t algo;                      /**<Algo struct*/
@@ -329,7 +358,7 @@ hipblasStatus_t hipblasLtGetArchName(char** archName);
  * opaque structure holding the hipBLASLt library context. It allocates light
  * hardware resources on the host and device, and must be called prior to making
  * any other hipBLASLt library calls. The hipBLASLt library context is tied to
- * the current CUDA device. To use the library on multiple devices, one
+ * the current ROCm device. To use the library on multiple devices, one
  * hipBLASLt handle should be created for each device.
  *
  *  @param[out]
@@ -687,7 +716,8 @@ hipblasStatus_t hipblasLtMatmulPreferenceGetAttribute(hipblasLtMatmulPreference_
  *  This function retrieves the possible algorithms for the matrix multiply
  * operation hipblasLtMatmul() function with the given input matrices A, B and
  * C, and the output matrix D. The output is placed in heuristicResultsArray[]
- * in the order of increasing estimated compute time.
+ * in the order of increasing estimated compute time. Note that the wall duration
+ * increases if the requestedAlgoCount increases.
  *
  *  @param[in]
  *  handle                  Pointer to the allocated hipBLASLt handle for the
@@ -843,40 +873,56 @@ hipblasStatus_t hipblasLtMatrixTransformDescSetAttribute( //
     const void*                              buf,
     size_t                                   sizeInBytes);
 
-/*! Get matrix transform operation descriptor attribute.
+/*! \ingroup library_module
+ *  \brief Matrix transform operation getter
+ *  \details Get matrix transform operation descriptor attribute.
  *
- * \param[in]  transformDesc  The descriptor
- * \param[in]  attr           The attribute
- * \param[out] buf            memory address containing the new value
- * \param[in]  sizeInBytes    size of buf buffer for verification (in bytes)
- * \param[out] sizeWritten    only valid when return value is CUBLAS_STATUS_SUCCESS. If sizeInBytes is non-zero: number
+ * @param[in]  transformDesc  The descriptor
+ * @param[in]  attr           The attribute
+ * @param[out] buf            memory address containing the new value
+ * @param[in]  sizeInBytes    size of buf buffer for verification (in bytes)
+ * @param[out] sizeWritten    only valid when return value is HIPBLAS_STATUS_SUCCESS. If sizeInBytes is non-zero: number
  * of bytes actually written, if sizeInBytes is 0: number of bytes needed to write full contents
  *
- * \retval     HIPBLAS_STATUS_INVALID_VALUE  if sizeInBytes is 0 and sizeWritten is NULL, or if  sizeInBytes is non-zero
+ * \retval HIPBLAS_STATUS_INVALID_VALUE  if sizeInBytes is 0 and sizeWritten is NULL, or if  sizeInBytes is non-zero
  *                                          and buf is NULL or sizeInBytes doesn't match size of internal storage for
  *                                          selected attribute
- * \retval     HIPBLAS_STATUS_SUCCESS        if attribute's value was successfully written to user memory
+ * \retval HIPBLAS_STATUS_SUCCESS        if attribute's value was successfully written to user memory
  */
 HIPBLASLT_EXPORT
-hipblasStatus_t hipblasLtMatrixTransformDescGetAttribute( //
-    hipblasLtMatrixTransformDesc_t           transformDesc,
-    hipblasLtMatrixTransformDescAttributes_t attr,
-    void*                                    buf,
-    size_t                                   sizeInBytes,
-    size_t*                                  sizeWritten);
+hipblasStatus_t
+    hipblasLtMatrixTransformDescGetAttribute(hipblasLtMatrixTransformDesc_t           transformDesc,
+                                             hipblasLtMatrixTransformDescAttributes_t attr,
+                                             void*                                    buf,
+                                             size_t                                   sizeInBytes,
+                                             size_t*                                  sizeWritten);
 
-/*! Matrix layout conversion helper (C = alpha * op(A) + beta * op(B))
+/*! \ingroup library_module
+ *  \brief Matrix layout conversion helper
+ *  \details
+ *   Matrix layout conversion helper (C = alpha * op(A) + beta * op(B)),
+ * can be used to change memory order of data or to scale and shift the values.
+ * @param[in]  lightHandle   Pointer to the allocated hipBLASLt handle for the
+ * hipBLASLt context. See \ref hipblasLtHandle_t .
+ * @param[in]  transformDesc Pointer to allocated matrix transform descriptor.
+ * @param[in]  alpha         Pointer to scalar alpha, either pointer to host or device address.
+ * @param[in]  A             Pointer to matrix A, must be pointer to device address.
+ * @param[in]  Adesc         Pointer to layout for input matrix A.
+ * @param[in]  beta          Pointer to scalar beta, either pointer to host or device address.
+ * @param[in]  B             Pointer to layout for matrix B, must be pointer to device address
+ * @param[in]  Bdesc         Pointer to layout for inputmatrix B.
+ * @param[in]  C             Pointer to matrix C, must be pointer to device address
+ * @param[out] Cdesc         Pointer to layout for output matrix C.
+ * @param[in] stream         The HIP stream where all the GPU work will be submitted.
  *
- * Can be used to change memory order of data or to scale and shift the values.
- *
- * \retval     HIPBLAS_STATUS_NOT_INITIALIZED   if hipBLASLt handle has not been initialized
- * \retval     HIPBLAS_STATUS_INVALID_VALUE     if parameters are in conflict or in an impossible configuration; e.g.
+ * \retval HIPBLAS_STATUS_NOT_INITIALIZED   if hipBLASLt handle has not been initialized
+ * \retval HIPBLAS_STATUS_INVALID_VALUE     if parameters are in conflict or in an impossible configuration; e.g.
  *                                              when A is not NULL, but Adesc is NULL
- * \retval     HIPBLAS_STATUS_NOT_SUPPORTED     if current implementation on selected device doesn't support configured
+ * \retval HIPBLAS_STATUS_NOT_SUPPORTED     if current implementation on selected device doesn't support configured
  *                                              operation
- * \retval     HIPBLAS_STATUS_ARCH_MISMATCH     if configured operation cannot be run using selected device
- * \retval     HIPBLAS_STATUS_EXECUTION_FAILED  if cuda reported execution error from the device
- * \retval     HIPBLAS_STATUS_SUCCESS           if the operation completed successfully
+ * \retval HIPBLAS_STATUS_ARCH_MISMATCH     if configured operation cannot be run using selected device
+ * \retval HIPBLAS_STATUS_EXECUTION_FAILED  if HIP reported execution error from the device
+ * \retval HIPBLAS_STATUS_SUCCESS           if the operation completed successfully
  */
 HIPBLASLT_EXPORT
 hipblasStatus_t hipblasLtMatrixTransform(hipblasLtHandle_t              lightHandle,
